/* boot.c - load and bootstrap a kernel */
/*
 *  GRUB  --  GRand Unified Bootloader
 *  Copyright (C) 1999,2000,2001,2002,2003,2004,2005  Free Software Foundation, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/*
 *  Modification History
 *  --------------------
 *  2013-09-17:  Implement verify_image() to verify bzImage with signature.
 */

#ifdef GRUB_ENABLE_VERIFIED_BOOT

#include "boot_verified-boot.h"
#include "boot_rsa-sha.c"

static void
sleep (int secs)
{
    unsigned long tmo = currticks () + secs;
    while (currticks () < tmo)
        ;
}

static void
count_wait (int secs)
{
    int remain = secs;

    printf ("\n");
    do {
        printf ("\rGRUB will move forward if not any key pressed in (%d) seconds ", remain);
        sleep (1 * SLEEP_MULTI_FACTOR);
        -- remain;
    } while (remain > 0 && checkkey() == -1);
    printf ("\r                                                                         \n");

    if (checkkey() != -1)
        getkey();
}

/*=============================================================================
 * Dump buffer for debug
 *============================================================================*/
static int
debug_dump_buffer(unsigned char *bufname, unsigned char *buf, int len)
{
#ifdef DEBUG_VERIFIED_BOOT
    int i = 0;
    grub_printf("\n%s[0x%x][%d]:\n", bufname, buf, len);
    for(i=0; i<len; i++){
        grub_printf("%x ", buf[i]);
        if(0 == ((i+1)%16)) grub_printf("\n");
    }
    grub_printf("\n");
    count_wait(60);
#endif

    return 0;
}

/*=============================================================================
 * Decode a base64-formatted buffer
 *============================================================================*/
static const unsigned char base64_dec_map[128] = {
    127, 127, 127, 127, 127, 127, 127, 127, 127, 127,
    127, 127, 127, 127, 127, 127, 127, 127, 127, 127,
    127, 127, 127, 127, 127, 127, 127, 127, 127, 127,
    127, 127, 127, 127, 127, 127, 127, 127, 127, 127,
    127, 127, 127,  62, 127, 127, 127,  63,  52,  53,
     54,  55,  56,  57,  58,  59,  60,  61, 127, 127,
    127,  64, 127, 127, 127,   0,   1,   2,   3,   4,
      5,   6,   7,   8,   9,  10,  11,  12,  13,  14,
     15,  16,  17,  18,  19,  20,  21,  22,  23,  24,
     25, 127, 127, 127, 127, 127, 127,  26,  27,  28,
     29,  30,  31,  32,  33,  34,  35,  36,  37,  38,
     39,  40,  41,  42,  43,  44,  45,  46,  47,  48,
     49,  50,  51, 127, 127, 127, 127, 127
};

static int
base64_decode(unsigned char *src, int slen,
              unsigned char *dst, int *dlen)
{
    int i, n;
    unsigned int j, x;
    unsigned char *p;

    for(i = j = n = 0; i < slen; i++) {
        if(( slen - i ) >= 2 && src[i] == '\r' && src[i + 1] == '\n')
            continue;

        if(src[i] == '\n')
            continue;

        if(src[i] == '=' && ++j > 2)
            return( ERR_BASE64_INVALID_CHARACTER );

        if(src[i] > 127 || base64_dec_map[src[i]] == 127)
            return (ERR_BASE64_INVALID_CHARACTER);

        if(base64_dec_map[src[i]] < 64 && j != 0)
            return (ERR_BASE64_INVALID_CHARACTER);

        n++;
    }

    if(n == 0)
        return 0;

    n = ((n * 6) + 7) >> 3;

    if(*dlen < n) {
        *dlen = n;
        return (ERR_BASE64_BUFFER_TOO_SMALL);
    }

    for(j = 3, n = x = 0, p = dst; i > 0; i--, src++){
        if(*src == '\r' || *src == '\n')
            continue;

        j -= (base64_dec_map[*src] == 64);
        x  = (x << 6) | (base64_dec_map[*src] & 0x3F);

        if(++n == 4){
            n = 0;
            if(j > 0) *p++ = (unsigned char)(x >> 16);
            if(j > 1) *p++ = (unsigned char)(x >>  8);
            if(j > 2) *p++ = (unsigned char)(x      );
        }
    }

    *dlen = p - dst;

    return 0;
}

/*=============================================================================
 * Functions to parse ASN1 element
 *===========================================================================*/
static int
asn1_get_len(unsigned char **p, unsigned char *end, int *len)
{
    if((end - *p) < 1)
        return( ERR_ASN1_OUT_OF_DATA );

    if((**p & 0x80) == 0)
        *len = *(*p)++;
    else {
        switch(**p & 0x7F) {
            case 1:
                if((end - *p) < 2 )
                    return (ERR_ASN1_OUT_OF_DATA);
                *len = (*p)[1];
                (*p) += 2;
                break;
            case 2:
                if((end - *p) < 3)
                    return (ERR_ASN1_OUT_OF_DATA);
                *len = ((*p)[1] << 8) | (*p)[2];
                (*p) += 3;
                break;
            case 3:
                if((end - *p) < 4)
                    return (ERR_ASN1_OUT_OF_DATA);
                *len = ((*p)[1] << 16) | ((*p)[2] << 8) | (*p)[3];
                (*p) += 4;
                break;
            case 4:
                if((end - *p) < 5)
                    return (ERR_ASN1_OUT_OF_DATA);
                *len = ((*p)[1] << 24) | ((*p)[2] << 16) | ((*p)[3] << 8) | (*p)[4];
                (*p) += 5;
                break;
            default:
                return (ERR_ASN1_INVALID_LENGTH);
        }
    }

    if(*len > (int)(end - *p))
        return (ERR_ASN1_OUT_OF_DATA);

    return 0;
}

/* "tag + len, without data*/
static int
asn1_get_tag(unsigned char **p, unsigned char *end, int *len, int tag)
{
    if((end - *p) < 1)
        return (ERR_ASN1_OUT_OF_DATA);

    if( **p != tag )
        return (ERR_ASN1_UNEXPECTED_TAG);

    (*p)++;

    return (asn1_get_len(p, end, len));
}

/* 02 len value */
static int
asn1_get_int(unsigned char **p, unsigned char *end, int *val)
{
    int ret, len;

    if(0 != (ret = asn1_get_tag(p, end, &len, ASN1_INTEGER)))
        return ret;

    if(len > sizeof(int) || (**p & 0x80) != 0)
        return (ERR_ASN1_INVALID_LENGTH);

    *val = 0;
    while(len-- > 0){
        *val = (*val << 8) | **p;
        (*p)++;
    }

    return 0;
}

/* Parse asn1 to mpi*/
static int
asn1_get_mpi(unsigned char **p, unsigned char *end, mpi *X)
{
    int ret, len;

    if(0 != (ret = asn1_get_tag(p, end, &len, ASN1_INTEGER)))
        return ret;

    ret = mpi_read_binary(X, *p, len);

    *p += len;

    return ret;
}

/* Version  ::=  INTEGER  { v1(0), v2(1), v3(2) } */
static int
x509_get_version( unsigned char **p, unsigned char *end, int *ver )
{
    int ret, len;

    if(0 != (ret = asn1_get_tag(p, end, &len, ASN1_CONTEXT_SPECIFIC | ASN1_CONSTRUCTED | 0))) {
        if(ret == ERR_ASN1_UNEXPECTED_TAG) {
            *ver = 0;
            return 0;
        }
        return ret;
    }

    end = *p + len;
    if(0 != (ret = asn1_get_int(p, end, ver)))
        return (ERR_X509_CERT_INVALID_VERSION + ret);

    if(*p != end)
        return (ERR_X509_CERT_INVALID_VERSION + ERR_ASN1_LENGTH_MISMATCH );

    return 0;
}

/* CertificateSerialNumber  ::=  INTEGER */
static int
x509_get_serial(unsigned char **p, unsigned char *end, x509_data_buf *serial)
{
    int ret, tag;

    if((end - *p) < 1)
        return (ERR_X509_CERT_INVALID_SERIAL + ERR_ASN1_OUT_OF_DATA);

    tag = *(*p) ++;
    if(tag != (ASN1_CONTEXT_SPECIFIC | ASN1_PRIMITIVE | 2) &&
       tag != ASN1_INTEGER)
        return( ERR_X509_CERT_INVALID_SERIAL + ERR_ASN1_UNEXPECTED_TAG );

    if(0 != (ret = asn1_get_len(p, end, &(serial->len))))
        return (ERR_X509_CERT_INVALID_SERIAL + ret);

    serial->buf = *p;
    *p += serial->len;

    return 0;
}

/* AlgorithmIdentifier  ::=  SEQUENCE  {
 *   algorithm               OBJECT IDENTIFIER,
 *   parameters              ANY DEFINED BY algorithm OPTIONAL } */
static int
x509_get_alg(unsigned char **p, unsigned char *end, x509_data_buf *alg)
{
    int ret, len;

    if(0 != (ret = asn1_get_tag(p, end, &len, ASN1_CONSTRUCTED | ASN1_SEQUENCE )))
        return (ERR_X509_CERT_INVALID_ALG + ret );

    end = *p + len;
    if(0 != (ret = asn1_get_tag(p, end, &(alg->len), ASN1_OID)))
        return (ERR_X509_CERT_INVALID_ALG + ret);

    alg->buf = *p;
    *p += alg->len;

    /* Assume the algorithm parameters must be NULL */
    if(*p != end) {
        if(0 != (ret = asn1_get_tag( p, end, &len, ASN1_NULL )))
            return (ERR_X509_CERT_INVALID_ALG + ret);

        if( *p != end )
            return (ERR_X509_CERT_INVALID_ALG + ERR_ASN1_LENGTH_MISMATCH);
    }

    return 0;
}

/* AttributeTypeAndValue ::= SEQUENCE {
 *   type     AttributeType,
 *   value    AttributeValue }
 *
 * AttributeType ::= OBJECT IDENTIFIER
 *
 * AttributeValue ::= ANY DEFINED BY AttributeType */
static int
x509_get_attr_type_value(unsigned char **p, unsigned char *end, x509_name *out)
{
    int ret, len;

    if(0 != (ret = asn1_get_tag(p, end, &len, ASN1_CONSTRUCTED | ASN1_SEQUENCE)))
        return (ERR_X509_CERT_INVALID_NAME + ret);

    if(0 != (ret = asn1_get_tag(p, end, &len, ASN1_OID)))
        return (ERR_X509_CERT_INVALID_NAME + ret);

    *p += len;
    if((end - *p ) < 1)
        return (ERR_X509_CERT_INVALID_NAME + ERR_ASN1_OUT_OF_DATA);

    if(**p != ASN1_BMP_STRING && **p != ASN1_UTF8_STRING &&
       **p != ASN1_T61_STRING && **p != ASN1_PRINTABLE_STRING &&
       **p != ASN1_IA5_STRING && **p != ASN1_UNIVERSAL_STRING)
        return (ERR_X509_CERT_INVALID_NAME + ERR_ASN1_UNEXPECTED_TAG);

    (*p)++;
    if(0 != (ret = asn1_get_len(p, end, &len)))
        return (ERR_X509_CERT_INVALID_NAME + ret);

    out->val.buf = *p;
    out->val.len = len;
    *p += len;

    return 0;
}

/* RelativeDistinguishedName ::=
 *   SET OF AttributeTypeAndValue */
static int
x509_get_name(unsigned char **p, unsigned char *end, x509_name *cur)
{
    int ret, len;
    unsigned char *end2;
    x509_name *use;

    if(0 != (ret = asn1_get_tag(p, end, &len, ASN1_CONSTRUCTED | ASN1_SET )))
        return (ERR_X509_CERT_INVALID_NAME + ret);

    end2 = end;
    end  = *p + len;
    use = cur;

    do {
        if(0 != (ret = x509_get_attr_type_value(p, end, use)))
            return ret;

        /* More than one attribute in one set*/
        if(*p != end){
            if(use->next == NULL)
                return (ERR_X509_NAME_SPACE_FULL);
            use = use->next;
        }
    }
    while( *p != end );

    /* recurse until end of SEQUENCE is reached */
    if(*p == end2)
        return 0;

    if(cur->next == NULL)
        return (ERR_X509_NAME_SPACE_FULL);

    return x509_get_name(p, end2, cur->next);
}


/* Time ::= CHOICE {
 *   utcTime        UTCTime,
 *   generalTime    GeneralizedTime }*/
static int
x509_get_time(unsigned char **p, unsigned char *end, x509_time *time)
{
    int ret, len;
    char date[64];
    unsigned char tag;

    if((end - *p) < 1 )
        return (ERR_X509_CERT_INVALID_DATE + ERR_ASN1_OUT_OF_DATA);

    tag = *(*p)++;

    if( 0 != (ret = asn1_get_len(p, end, &len)))
        return (ERR_X509_CERT_INVALID_DATE + ret);

    if(len > (sizeof(date) - 1))
        return ERR_X509_CERT_INVALID_DATE;

    memset(date, 0, sizeof(date));
    memcpy(date, *p, len);
    *p += len;

    /*no sscanf in grub stage2*/
    time->year = ((date[0] - 0x30) * 10) + (date[1] - 0x30);
    time->mon  = ((date[2] - 0x30) * 10) + (date[3] - 0x30);
    time->day  = ((date[4] - 0x30) * 10) + (date[5] - 0x30);
    time->hour = ((date[6] - 0x30) * 10) + (date[7] - 0x30);
    time->min  = ((date[8] - 0x30) * 10) + (date[9] - 0x30);
    time->sec  = ((date[10]- 0x30) * 10) + (date[11]- 0x30);

    if(tag == ASN1_UTC_TIME) {
        time->year += 100 * (time->year < 50);
        time->year += 1900;
        return 0;
    }
    else if(tag == ASN1_GENERALIZED_TIME) {
        return( 0 );
    }
    else
        return (ERR_X509_CERT_INVALID_DATE + ERR_ASN1_UNEXPECTED_TAG);
}

/* Validity ::= SEQUENCE {
 *   notBefore      Time,
 *   notAfter       Time } */
static int
x509_get_dates(unsigned char **p, unsigned char *end,
        x509_time *from, x509_time *to)
{
    int ret, len;

    if(0 != (ret = asn1_get_tag(p, end, &len, ASN1_CONSTRUCTED | ASN1_SEQUENCE )))
        return (ERR_X509_CERT_INVALID_DATE + ret);

    end = *p + len;

    if(0 != (ret = x509_get_time(p, end, from)))
        return ret;

    if(0 != (ret = x509_get_time(p, end, to)))
        return ret;

    if( *p != end )
        return (ERR_X509_CERT_INVALID_DATE + ERR_ASN1_LENGTH_MISMATCH);

    return 0;
}

/* This function can also be used to parse public key,
 * if it is converted from base64 to ASN1 firstly.
 *
 * SubjectPublicKeyInfo  ::=  SEQUENCE  {
 *   algorithm            AlgorithmIdentifier,
 *   subjectPublicKey     BIT STRING } */
static int
x509_get_pubkey(unsigned char **p, unsigned char *end, x509_pubkey *pubkey)
{
    int ret, len;
    unsigned char *end_pk;

    debug_dump_buffer("pubkey", *p, end - *p);

    if(0 != (ret = asn1_get_tag(p, end, &len, ASN1_CONSTRUCTED | ASN1_SEQUENCE )))
        return (ERR_X509_CERT_INVALID_FORMAT + ret);

    if(0 != (ret = x509_get_alg(p, end, &(pubkey->pubAlg))))
        return ret;

    /* Only RSA public keys handled at this time */
    if(pubkey->pubAlg.len != 9 || memcmp(pubkey->pubAlg.buf, OID_PKCS1_RSA, 9) != 0)
        return (ERR_X509_CERT_UNSUPPORT_PKALG);

    if(0 != (ret = asn1_get_tag(p, end, &len, ASN1_BIT_STRING)))
        return (ERR_X509_CERT_INVALID_PUBKEY + ret);

    if((end - *p) < 1)
        return (ERR_X509_CERT_INVALID_PUBKEY + ERR_ASN1_OUT_OF_DATA);

    end_pk = *p + len;

    if( *(*p)++ != 0 )
        return (ERR_X509_CERT_INVALID_PUBKEY);

    /* RSAPublicKey ::= SEQUENCE {
     *     modulus           INTEGER,  -- n
     *     publicExponent    INTEGER   -- e } */
    if(0 != (ret = asn1_get_tag(p, end_pk, &len, ASN1_CONSTRUCTED | ASN1_SEQUENCE)))
        return (ERR_X509_CERT_INVALID_PUBKEY + ret);

    if(*p + len != end_pk)
        return (ERR_X509_CERT_INVALID_PUBKEY + ERR_ASN1_LENGTH_MISMATCH);


    /* Initialize publick key firstly*/
    rsa_init((rsa_context *)&pubkey->rsa, RSA_PKCS_V15, 0);

    debug_dump_buffer("pubkeyN+E", *p, end_pk - *p);
    if(0 != (ret = asn1_get_mpi(p, end_pk, (mpi *)&(pubkey->rsa.N))) ||
       0 != (ret = asn1_get_mpi(p, end_pk, (mpi *)&(pubkey->rsa.E))))
        return (ERR_X509_CERT_INVALID_PUBKEY + ret);

    if(*p != end)
        return (ERR_X509_CERT_INVALID_PUBKEY + ERR_ASN1_LENGTH_MISMATCH);

    pubkey->rsa.len = mpi_size(&(pubkey->rsa.N));
    GRUB_INFO("pubkey len: %d\n", pubkey->rsa.len);
    GRUB_INFO("pubkey N len: %d\n", pubkey->rsa.N.n);
    GRUB_INFO("pubkey E len: %d\n", pubkey->rsa.E.n);
    debug_dump_buffer("pubkeyN", (unsigned char *)pubkey->rsa.N.p, 128);
    debug_dump_buffer("pubkeyE", (unsigned char *)pubkey->rsa.E.p, 3);
    return 0;
}

/* signatureValue       BIT STRING */
static int
x509_get_sig(unsigned char **p, unsigned char *end, x509_data_buf *sign)
{
    int ret, len;

    if((end - *p) < 1)
        return (ERR_X509_CERT_INVALID_SIGNATURE + ERR_ASN1_OUT_OF_DATA );

    if(0 != (ret = asn1_get_tag(p, end, &len, ASN1_BIT_STRING)))
        return (ERR_X509_CERT_INVALID_SIGNATURE + ret);

    if(--len < 1 || *(*p)++ != 0)
        return (ERR_X509_CERT_INVALID_SIGNATURE);

    sign->len = len;
    sign->buf = *p;

    *p += len;
    return 0;
}

/*=============================================================================
 * Get pem entry from pem file buffer.
 *===========================================================================*/
static int
pem_get_entry(unsigned char *pembuf,
        const char *header, const char *footer,
        unsigned char *outbuf, int *outlen)
{
    unsigned char *begin, *end;
    int base64_len;

    /* Find base64 entry data*/
    if(NULL == (begin = strstr(pembuf, header)))
        return (ERR_PEM_NO_BEGIN);

    if(NULL == (end = strstr(pembuf, footer)))
        return (ERR_PEM_NO_END);

    /* Adjust begin pointer*/
    begin += strlen(header);
    if( *begin == '\r' ) begin++;
    if( *begin == '\n' ) begin++;
    else return (ERR_PEM_CHARACTER);

    if (end <= begin)
        return (ERR_PEM_INDEX_POSITION);

    base64_len = (int)(end - begin);
    return base64_decode(begin, end-begin, outbuf, outlen);
}

/*=============================================================================
 * Parse a X.509 certificate in ANS1 format
 *===========================================================================*/
static int
x509parse_crt_der(x509_cert *crt, unsigned char *buf, int buflen)
{
    int ret, i, len;
    unsigned char *p = buf, *end = buf + buflen, *crt_end;

    if(crt == NULL || buf == NULL)
        return (ERR_X509_CERT_INVALID_INPUT);

    /* Certificate  ::=  SEQUENCE  {
     *      tbsCertificate       TBSCertificate,
     *      signatureAlgorithm   AlgorithmIdentifier,
     *      signatureValue       BIT STRING  } */
    if(0 != (ret = asn1_get_tag(&p, end, &len, ASN1_CONSTRUCTED | ASN1_SEQUENCE ))) {
        return (ERR_X509_CERT_INVALID_FORMAT + ret);
    }

    if(len != (int)(end - p)) {
        return (ERR_X509_CERT_INVALID_TOP_LEN + ret);
    }

    crt_end = p + len;

    /* TBSCertificate  ::=  SEQUENCE  { } */
    crt->tbs.buf = p;

    if(0 != (ret = asn1_get_tag(&p, end, &len, ASN1_CONSTRUCTED | ASN1_SEQUENCE ))) {
        return (ERR_X509_CERT_INVALID_FORMAT + ret);
    }

    end = p + len;  /*change end to tbs end*/
    crt->tbs.len = end - crt->tbs.buf;

    /* Version  ::=  INTEGER  {  v1(0), v2(1), v3(2)  } */
    if(0 != (ret = x509_get_version(&p, end, &(crt->version)))) {
        return ret;
    }

    /* CertificateSerialNumber  ::=  INTEGER */
    if(0 != (ret = x509_get_serial(&p, end, &(crt->serial)))) {
       return ret;
    }
    debug_dump_buffer("serialNumber", crt->serial.buf, crt->serial.len);

    /* AlgorithmIdentifier  ::=  SEQUENCE  */
    if(0 != (ret = x509_get_alg(&p, end, &(crt->tbsAlg)))) {
        return ret;
    }

    if( ++ (crt->version) > 3) {
        return (ERR_X509_CERT_UNKNOWN_VERSION);
    }

    /*if(0 != memcmp((const char *)crt->tbsAlg.buf, OID_PKCS1_SHA256, crt->tbsAlg.len)){
        return (ERR_X509_CERT_INVALID_ALG);
    }    */

    /* Issuer               Name */
    for(i=0; i<CNT_SUBJ_NAME-1; i++) {
        crt->issuer[i].next = &(crt->issuer[i+1]);
    }
    crt->issuer[i].next=NULL;

    if(0 != (ret = asn1_get_tag(&p, end, &len, ASN1_CONSTRUCTED | ASN1_SEQUENCE ))) {
        return (ERR_X509_CERT_INVALID_FORMAT + ret);
    }

    if(0 != (ret = x509_get_name(&p, p + len, &(crt->issuer[0])))){
        return ret ;
    }

    /* Validity ::= SEQUENCE {
     *     notBefore      Time,
     *     notAfter       Time } */
    if(0 != (ret = x509_get_dates(&p, end, &crt->valid_from, &crt->valid_to ))) {
        return ret;
    }

    /* Subject              Name */
    for(i=0; i<CNT_SUBJ_NAME-1; i++) {
        crt->subject[i].next = &(crt->subject[i+1]);
    }
    crt->subject[i].next=NULL;

    if(0 != (ret = asn1_get_tag(&p, end, &len, ASN1_CONSTRUCTED | ASN1_SEQUENCE ))) {
        return (ERR_X509_CERT_INVALID_FORMAT + ret);
    }

    if(0 != (ret = x509_get_name(&p, p + len, &(crt->subject[0])))){
        return ret ;
    }

    /* SubjectPublicKeyInfo  ::=  SEQUENCE */
    if(0 != (ret = x509_get_pubkey(&p, end, &crt->pubkey))) {
        return ret;
    }

    /* Parse V2 V3 UID and extend here, but not support now*/
    if(crt->version == 2 || crt->version == 3) {
        GRUB_DBG("Ignore x509 V%d special items.\n", crt->version);
        p = end; /*Jump over others about V2 and V3 in tbs.*/
        /*return ERR_X509_CERT_INVALID_VERSION;*/
    }

    /* End of tbs,  change end to certificate end.*/
    end = crt_end;

    /* signatureAlgorithm   AlgorithmIdentifier,
     * signatureValue       BIT STRING */
    if(0 != (ret = x509_get_alg(&p, end, &crt->signAlg))) {
        return( ret );
    }

    if(0 != (ret = x509_get_sig(&p, end, &crt->sign))) {
        return ret;
    }
    debug_dump_buffer("signbuf", crt->sign.buf, crt->sign.len);

    if(p != end) {
        return (ERR_X509_CERT_INVALID_FORMAT + ERR_ASN1_LENGTH_MISMATCH);
    }

    return 0;
}
/*=============================================================================
 * Parse certificate and get hash value, signature, public key.
 *===========================================================================*/
static int
x509parse_cert(x509_cert *crt, unsigned char *filebuf,
        unsigned char *certbuf, int *certlen)
{
    int ret;

    if(0 != (ret = pem_get_entry(filebuf,
            (const char *)"-----BEGIN CERTIFICATE-----",
            (const char *)"-----END CERTIFICATE-----",
            certbuf, certlen))){
        return ret;
    }

    if(0 != (ret = x509parse_crt_der(crt, certbuf, *certlen))) {
        return ret;
    }

    return 0;
}

/*=============================================================================
 * Verify certification by another parent certification.
 *============================================================================*/
static int
verify_cert(x509_cert *cert, x509_cert *rootcert)
{
    unsigned char hashcalbuf[SIZE_SHA256_HASH_HEX];
    unsigned char hashdecryptbuf[SIZE_RAS_SIGN_HEX];
    int ret;

    /* Decrypt sha256 from cert sign with pubkey in root cert */
    if(0 != (ret = rsa_public(&(rootcert->pubkey.rsa), cert->sign.buf, hashdecryptbuf))){
        GRUB_ERR("Decypt cert signature error!(%d)\n", ret);
        return ret;
    }
    debug_dump_buffer("decrypted cert tbs sha256 hash", hashdecryptbuf, rootcert->pubkey.rsa.len);

    /* Calculate sha256 hash value of cert tbs. */
    if(0 != (ret = cal_sha256_hash(NULL, 0, cert->tbs.buf, cert->tbs.len, hashcalbuf))) {
        GRUB_ERR("Calulate sha256 hash of cert tbs error!\n");
        return ret;
    }
    debug_dump_buffer("cal cert tbs sha256 hash", hashcalbuf, SIZE_SHA256_HASH_HEX);

    /* Compare sha256 hash value */
    if(0 != memcmp((hashdecryptbuf + rootcert->pubkey.rsa.len - SIZE_SHA256_HASH_HEX),
                hashcalbuf, SIZE_SHA256_HASH_HEX)){
        GRUB_ERR("sha256 hash value in certificate dismatch!\n");
        return ERR_VERIFYBOOT_CERT_HASH;
    }

    return 0;
}

/*=============================================================================
 * Read signature at the end of bzImage.
 * "WRS\0" + Lenght of signature(header + hash + certificate + "WRS\0" + Length)
 *============================================================================*/
static int
verify_image_read_signature(image_sign *imagesign)
{
    int ret, filelen, signlen;
    unsigned char tmpbuf[8];
    unsigned char magic[4]={'W', 'R', 'S', '\0'};

    if(!grub_open(imagesign->filename)) {
        return (ERR_VERIFYBOOT_OPEN_FAIL);
    }
    filelen = filemax;
    GRUB_INFO("filelen=%d(0x%x)\n", filelen, filelen);

    grub_seek(filelen - 8);
    if(8 != grub_read(tmpbuf, 8)) {
        grub_close();
        return (ERR_VERIFYBOOT_READ_FAIL);
    }
    debug_dump_buffer("magic&len", tmpbuf, 8); 

    if(0 != memcmp(tmpbuf, magic, 4)) {
        grub_close();
        return (ERR_VERIFYBOOT_MAGIC_NUM);
    }

    signlen = (((int)tmpbuf[4]) << 24) | (((int)tmpbuf[5]) << 16) |
             (((int)tmpbuf[6]) << 8) | (int)tmpbuf[7];
    if(signlen > imagesign->signlen){
        grub_close();
        return (ERR_VERIFYBOOT_SIGN_LEN);
    }
    GRUB_INFO("signlen=%d(0x%x)\n", signlen, signlen);

    grub_seek(filelen - signlen);
    if(signlen != grub_read(imagesign->signbuf, signlen)){
        grub_close();
        return (ERR_VERIFYBOOT_READ_FAIL);
    }

    imagesign->filelen = filelen;
    imagesign->signlen = signlen;
    grub_close();
    return 0;
}

/*=============================================================================
 * Show all needed information in bzImage signature and certificates.
 *============================================================================*/
#ifdef DEBUG_VERIFIED_BOOT
static void
show_cert_serial(unsigned char *prefix, x509_data_buf *serial, unsigned char *postfix)
{
    int i;

    grub_printf("%s0x", prefix);
    for(i = 1; i<serial->len; i++) {
        grub_printf("%x%x", (serial->buf[i] & 0xF0)>>4, (serial->buf[i] & 0x0F));
    }
    grub_printf("%s", postfix);
}

static void
show_cert_subj(unsigned char *prefix, x509_name *name, unsigned char *postfix)
{
    unsigned char tmpbuf[SIZE_X509_NAME_MAX+4];
    int tmplen = (name->val.len > SIZE_X509_NAME_MAX) ? \
                  SIZE_X509_NAME_MAX : name->val.len;

    memcpy(tmpbuf, name->val.buf, name->val.len);
    tmpbuf[name->val.len] = 0;
    grub_printf("%s%s%s", prefix, tmpbuf, postfix);
}

static void
show_cert_all(x509_cert *crt)
{
    grub_printf("Cert Version : %d\n", crt->version);
    show_cert_serial("Serial Number: ", &crt->serial, "\n");
    grub_printf("Issuer: ");
    show_cert_subj(" C=", &crt->issuer[0], ",");
    show_cert_subj("ST=", &crt->issuer[1], ",");
    show_cert_subj("L=", &crt->issuer[2], ",");
    show_cert_subj("O=", &crt->issuer[3], ",");
    show_cert_subj("OU=", &crt->issuer[4], ",\n");
    show_cert_subj("         CN=", &crt->issuer[5], "/");
    show_cert_subj("emailAdress=", &crt->issuer[6], "\n");
    grub_printf("Date Validity: %d/%d/%d ~ %d/%d/%d\n",
            crt->valid_from.mon, crt->valid_from.day, crt->valid_from.year,
            crt->valid_to.mon, crt->valid_to.day, crt->valid_to.year);
    grub_printf("Subject: ");
    show_cert_subj("C=", &crt->subject[0], ",");
    show_cert_subj("ST=", &crt->subject[1], ",");
    show_cert_subj("L=", &crt->subject[2], ",");
    show_cert_subj("O=", &crt->subject[3], ",");
    show_cert_subj("OU=", &crt->subject[4], ",\n");
    show_cert_subj("         CN=", &crt->subject[5], "/");
    show_cert_subj("emailAdress=", &crt->subject[6], "\n");
}
#endif

static void
verify_image_show_all(image_sign *imagesign, x509_cert *cert, x509_cert *rootcert)
{
#ifdef DEBUG_VERIFIED_BOOT
    grub_printf("SIGN INFO: VersionID=%X-%X\n", imagesign->header.mver,
                                                imagesign->header.sver);
    grub_printf("SIGN INFO: OwnerID=%X\n",  imagesign->header.ownerid);
    grub_printf("SIGN INFO: VendorID=%X\n", imagesign->header.vendorid);

    grub_printf("ROOT CERTIFICATE INFO ==================================\n");
    show_cert_all(rootcert);
    grub_printf("CERTIFICATE INFO =======================================\n");
    show_cert_all(cert);
    grub_printf("========================================================\n");
    count_wait(60);
#endif
}

/*=============================================================================
 * Verify kernel image by parsing the signature at the end of it.
 *============================================================================*/
static int
verify_image(char *kernelfile)
{
    int ret, rootcertlen, certlen;
    unsigned char imagesignbuf[SIZE_BZIMAGE_SIGN];
    unsigned char rootcertbuf[SIZE_CERT_ASN1];
    unsigned char certbuf[SIZE_CERT_ASN1];
    unsigned char hashcalbuf[SIZE_SHA256_HASH_HEX];
    unsigned char hashdecryptbuf[SIZE_RAS_SIGN_HEX];
    unsigned char *p;
    image_sign imagesign;
    x509_cert rootcert, cert;

    /* Read signature at the end of bzImage*/
    imagesign.filename = kernelfile;
    imagesign.signbuf = imagesignbuf;
    imagesign.signlen = SIZE_BZIMAGE_SIGN;
    if(0 != (ret = verify_image_read_signature(&imagesign))) {
        GRUB_DBG("verify_image_read_signature return: %d\n", ret);
        return ret;
    }

    p = imagesign.signbuf;

    /* Parse image singnature header(8 bytes)
     * MainVer    SubVer     OwnerID VendroID
     * TimaStamp0 TimaStamp1 AlgID   HashID*/
    memcpy(&imagesign.header, p, sizeof(image_header));
    p += sizeof(image_header);

    /* Parse sha256 hash signature*/
    imagesign.hashlen = (((int)p[0]) << 8) | (int)p[1];
    imagesign.hashbuf = p + 2;
    p += (imagesign.hashlen + 2);
    GRUB_INFO("image+header hashlen=%d\n", imagesign.hashlen);

    /* Parse certification file*/
    imagesign.certlen = (((int)p[0]) << 8) | (int)p[1];
    imagesign.certbuf = p + 2;
    imagesign.certbuf[imagesign.certlen] = '\0';
    p += (imagesign.certlen + 2);
    GRUB_INFO("certificate len=%d\n", imagesign.certlen);

    if(imagesign.signlen != (sizeof(image_header) +
                2 + imagesign.hashlen +
                2 + imagesign.certlen + 8)) {
        return (ERR_VERIFYBOOT_LEN_MISMATCH);
    }

    /* Verify certificate in bzImage signature by root certificate:
     * Root certificate is in header file, named "root_cert_buffer". */
    GRUB_INFO("\nLoading root certificate ...\n");
    rootcertlen = SIZE_CERT_ASN1;
    if(0 != (ret = x509parse_cert(&rootcert, root_cert_buffer,
                                   rootcertbuf, &rootcertlen))) {
        return (ERR_VERIFYBOOT_LOAD_CERT + ret);
    }
    GRUB_INFO("\nLoading signed certificate ...\n");
    certlen = SIZE_CERT_ASN1;
    if(0 != (ret = x509parse_cert(&cert, imagesign.certbuf,
                                   certbuf, &certlen))) {
        return (ERR_VERIFYBOOT_LOAD_CERT + ret);
        return -1;
    }
    if(0 != (ret = verify_cert(&cert, &rootcert))) {
        GRUB_ERR("Verify certificate error!(%d)\n", ret);
        return -1;
    }

    /* Show all the information for imagesign and certificates
     * Put it here because cal_sha256_hash to bzImage need some time,
     * so we have time to have a quick look at it.*/
    verify_image_show_all(&imagesign, &cert, &rootcert);

    /* Verify hash value for bzImage and signature header.*/
    if(0 != (ret = rsa_public(&(cert.pubkey.rsa), imagesign.hashbuf, hashdecryptbuf))) {
        GRUB_ERR("Decrypt bzImage sha256 hash error!(%d)\n", ret);
        return -1;
    }
    debug_dump_buffer("Decrypted bzImage sha256 hash", hashdecryptbuf, cert.pubkey.rsa.len);

    if(0 != (ret = cal_sha256_hash(imagesign.filename,
                    imagesign.filelen - imagesign.signlen + sizeof(image_header),
                    NULL, 0, hashcalbuf))){
        GRUB_ERR("Calulate sha256 hash of vendor public key error!\n");
        return -1;
    }
    debug_dump_buffer("cal bzImage sha256 hash", hashcalbuf, SIZE_SHA256_HASH_HEX);

    if(0 != memcmp(hashdecryptbuf + cert.pubkey.rsa.len - SIZE_SHA256_HASH_HEX,
                hashcalbuf, SIZE_SHA256_HASH_HEX)){
        GRUB_ERR("sha256 hash of vendor pubkey dismatch!\n");
        return -1;
    }

    return 0;
}

static int
verify_before_load_image(char *kernel)
{
    int ret;
    char  krnl_fn_buf[128];

    /*make sure "-----EMPTY CERTIFICATE-----" appear one time in final stage2 file.*/
    unsigned char *emptyid = "EMPTY CERTIFICATE";
    unsigned char *splitstr = "-----";

    memset(krnl_fn_buf, 0, sizeof(krnl_fn_buf));

    if(0 != memcmp(&root_cert_buffer[strlen(splitstr)], emptyid, strlen(emptyid))) {
        /* verified boot: verifiy the signature of bzImage*/
        grub_printf("\n########################################################");
        grub_printf("\n##                                                    ##");
        grub_printf("\n##               Verified Booting ...                 ##");
        grub_printf("\n##                                                    ##");
        grub_printf("\n########################################################\n");

        strncat(krnl_fn_buf, kernel, strstr(kernel, " ")-kernel+1);
        GRUB_DBG("kernel name: %s\n", krnl_fn_buf);
        GRUB_DBG("new name: %s\n", krnl_fn_buf);
        if(0 != (ret = verify_image(krnl_fn_buf))) {
            GRUB_ERR("Verify Result: %d\n", ret);
            grub_printf("\nError when verified booting.\n");
            grub_printf("\nPress any key to continue ...\n");
            getkey ();
            grub_reboot();
        }
        grub_printf("\n########################################################");
        grub_printf("\n##                                                    ##");
        grub_printf("\n##                 Verification OK                    ##");
        grub_printf("\n##                                                    ##");
        grub_printf("\n########################################################\n");
    }
    simple_free_all ();

    return ret;
}
#endif /*#ifdef GRUB_ENABLE_VERIFIED_BOOT*/

